;;; keytobuff.el --- Hotkeys for buffers.
;;
;; ~/share/emacs/pkg/keytobuff/keytobuff.el ---
;;
;; $Id: keytobuff.el,v 1.4 2010/05/21 20:47:58 harley Exp $
;;

;; Author:    Harley Gorrell <harley@panix.com>
;; URL:       http://www.mahalito.net/~harley/elisp/keytobuff.el
;; License:   GPL v2
;; Keywords:  emacs, hotkeys, windows, keys

;;; Commentary:
;; * Bind a key to 'keytobuff'
;;   Set the buffer for the with 'C-u key'.
;;   Press the key to bring the buffer to the current window.
;; * This is kind of like "screens.el", but without the hard
;;   limit of 10 screens or the fixed keybindings.
;; * There are probably other packages which do this, but I 
;;   didnt find them.

;;; History:
;; * 2004-03-13: package rename and clean up.

(defvar *keytobuff* nil
  "A mapping of keys to buffers.")

;;; Code:

(defun keytobuff-lookup (key)
  "Always return a record for KEY."
  (or (assoc key *keytobuff*)
      (let ((rec (list key nil)))
        (setq *keytobuff* (cons rec *keytobuff*))
        rec)))
;; (setcdr (keytobuff-lookup 1) (list 'foo))

;;;###autoload
(defun keytobuff (&optional arg key)
  "Prefix ARG set the buffer for KEY.  Otherwise goto the buffer.
A prefix of '1' records the window configuration, any other
prefix records the `current-buffer'."
  (interactive "P")
  (let ((rec (keytobuff-lookup (or key last-command-char))))
    (if arg
      ;; remember window config or buffer
      (progn
        (setcdr rec (list (if (= 1 (prefix-numeric-value arg))
                            (current-window-configuration)
                            (current-buffer) )))
        (message "Key set."))
      ;; goto the buffer
      (let ((info (cadr  rec)))
        (if (window-configuration-p info)
          (set-window-configuration info)
          (if (buffer-live-p info)
            (switch-to-buffer info)
            (message "Dead buffer."))
          (message "No buffer set."))))))

;;;###autoload
(defun keytobuff-set-keys (&rest keys)
  "Bind KEYS to keytobuff."
  (while keys
    (global-set-key (car keys) 'keytobuff)
    (setq keys (cdr keys))))
;; (keytobuff-set-keys [f6] [f7] [f8])

(defun keytobuff-clear ()
  "Clears the map of keys to buffers."
  (interactive)
  (setq *keytobuff* nil))
;; (keytobuff-clear)

(provide 'keytobuff)

;;; keytobuff.el ends here
