;;
;; ~/share/emacs/pkg/kjd/kjd-server.el ---
;;
;; $Id: kjd-server.el,v 1.4 2003/11/07 21:47:30 harley Exp $
;;

;; Commentary:
;; * Use a new frame when emacsclient connects.

;; make sure it is loaded for advice.
(require 'server)

;; Code:

(defvar kjd-server-open-frame-cnt 0
  "Make a new frame for the buffer in kjd-server-open-frame when CNT>0")

(defvar kjd-server-boring-buffer-regexp "^ ?\\*"
  "A regexp for the name of a boring buffer.")

(defun kjd-server-boring-buffer-p (bufname)
  "Non nil if the buffer is boring."
  (if (bufferp bufname)
      (setq bufname (buffer-name bufname)))
  (string-match kjd-server-boring-buffer-regexp bufname))

;; You would think advice could do this but when mutiple
;; files are sent, we need a per file action. This is it
(defun kjd-server-open-frame ()
  "A hook fuction to open the client buffer in a new frame."
  ;; This will still change the buffer shown in the current frame.
  ;; Rely on the advice to restore the original frame.
  (if (> kjd-server-open-frame 0)
      (let ((client-buffer (current-buffer)))
        (bury-buffer client-buffer)
        (switch-to-buffer-other-frame client-buffer)))
  (setq kjd-server-open-frame (1+ kjd-server-open-frame)))

(add-hook 'server-visit-hook 'kjd-server-open-frame)

(defun kjd-server-done-frame ()
  "A hook function to dispose of the current frame when done."
  (message "DONE")
  (if (cdr (frame-list)) ;; dont do the last one.
      (delete-frame)))
(add-hook 'server-done-hook 'kjd-server-done-frame)

;; when opening more than one file at a time with
;; emacsclient and the front buffer is boring, only one
;; emacsclient buffer will show.  
;; Could be fixed but does anyone care?
(defadvice server-visit-files (around kjd-server-visit activate)
  "Preserve the existing window state.
If another frame isnt opened, you wont see your server buffer."
  (let ((kjd-server-open-frame
         (if (kjd-server-boring-buffer-p (current-buffer)) 0 1)))
    (if kjd-server-open-frame
        (save-excursion ;; keep the layout
          ad-do-it)
      ;; dont keep the layout.
      ad-do-it)))


